@@ -27,6 +27,9 @@
 
 class TestFolderMan;
 class TestCfApiShellExtensionsIPC;
+class TestShareModel;
+class ShareTestHelper;
+class EndToEndTestHelper;
 
 namespace OCC {
 
@@ -71,6 +74,11 @@ public:
         ErrorNonEmptyFolder
     };
 
+    enum class GoodPathStrategy {
+        AllowOnlyNewPath,
+        AllowOverrideExistingPath,
+    };
+
     ~FolderMan() override;
     static FolderMan *instance();
 
@@ -83,7 +91,7 @@ public:
      */
     static void backwardMigrationSettingsKeys(QStringList *deleteKeys, QStringList *ignoreKeys);
 
-    const Folder::Map &map() const;
+    [[nodiscard]] const Folder::Map &map() const;
 
     /** Adds a folder for an account, ensures the journal is gone and saves it in the settings.
       */
@@ -144,7 +152,7 @@ public:
      *
      * @returns an empty string and PathValidityResult::Valid if it is allowed, or an error if it is not allowed
      */
-    QPair<PathValidityResult, QString> checkPathValidityForNewFolder(const QString &path, const QUrl &serverUrl = QUrl()) const;
+    [[nodiscard]] QPair<PathValidityResult, QString> checkPathValidityForNewFolder(const QString &path, const QUrl &serverUrl = QUrl()) const;
 
     /**
      * Attempts to find a non-existing, acceptable path for creating a new sync folder.
@@ -155,7 +163,7 @@ public:
      * subfolder of ~ would be a good candidate. When that happens \a basePath
      * is returned.
      */
-    QString findGoodPathForNewSyncFolder(const QString &basePath, const QUrl &serverUrl) const;
+    [[nodiscard]] QString findGoodPathForNewSyncFolder(const QString &basePath, const QUrl &serverUrl, GoodPathStrategy allowExisting) const;
 
     /**
      * While ignoring hidden files can theoretically be switched per folder,
@@ -163,13 +171,13 @@ public:
      * at once.
      * These helper functions can be removed once it's properly per-folder.
      */
-    bool ignoreHiddenFiles() const;
+    [[nodiscard]] bool ignoreHiddenFiles() const;
     void setIgnoreHiddenFiles(bool ignore);
 
     /**
      * Access to the current queue of scheduled folders.
      */
-    QQueue<Folder *> scheduleQueue() const;
+    [[nodiscard]] QQueue<Folder *> scheduleQueue() const;
 
     /**
      * Access to the currently syncing folder.
@@ -179,7 +187,7 @@ public:
      *
      * See also isAnySyncRunning()
      */
-    Folder *currentSyncFolder() const;
+    [[nodiscard]] Folder *currentSyncFolder() const;
 
     /**
      * Returns true if any folder is currently syncing.
@@ -187,7 +195,7 @@ public:
      * This might be a FolderMan-scheduled sync, or a externally
      * managed sync like a placeholder hydration.
      */
-    bool isAnySyncRunning() const;
+    [[nodiscard]] bool isAnySyncRunning() const;
 
     /** Removes all folders */
     int unloadAndDeleteAllFolders();
@@ -213,8 +221,8 @@ public:
     void setDirtyProxy();
     void setDirtyNetworkLimits();
 
-    /** opens a file with default app, if the file is present **/
-    void editFileLocally(const QString &userId, const QString &relPath, const QString &token);
+    /** removes current user from the share **/
+    void leaveShare(const QString &localFile);
 
 signals:
     /**
@@ -222,7 +230,7 @@ signals:
       *
       * Attention: The folder may be zero. Do a general update of the state then.
       */
-    void folderSyncStateChange(Folder *);
+    void folderSyncStateChange(OCC::Folder *);
 
     /**
      * Indicates when the schedule queue changes.
@@ -232,12 +240,12 @@ signals:
     /**
      * Emitted whenever the list of configured folders changes.
      */
-    void folderListChanged(const Folder::Map &);
+    void folderListChanged(const OCC::Folder::Map &);
 
     /**
      * Emitted once slotRemoveFoldersForAccount is done wiping
      */
-    void wipeDone(AccountState *account, bool success);
+    void wipeDone(OCC::AccountState *account, bool success);
 
 public slots:
 
@@ -261,16 +269,20 @@ public slots:
     void slotSyncOnceFileUnlocks(const QString &path);
 
     // slot to schedule an ETag job (from Folder only)
-    void slotScheduleETagJob(const QString &alias, RequestEtagJob *job);
+    void slotScheduleETagJob(const QString &alias, OCC::RequestEtagJob *job);
 
     /** Wipe folder */
-    void slotWipeFolderForAccount(AccountState *accountState);
+    void slotWipeFolderForAccount(OCC::AccountState *accountState);
+
+    void forceSyncForFolder(OCC::Folder *folder);
+
+    void removeE2eFiles(const AccountPtr &account) const;
 
 private slots:
-    void slotFolderSyncPaused(Folder *, bool paused);
+    void slotFolderSyncPaused(OCC::Folder *, bool paused);
     void slotFolderCanSyncChanged();
     void slotFolderSyncStarted();
-    void slotFolderSyncFinished(const SyncResult &);
+    void slotFolderSyncFinished(const OCC::SyncResult &);
 
     void slotRunOneEtagJob();
     void slotEtagJobDestroyed(QObject *);
@@ -279,15 +291,15 @@ private slots:
     void slotStartScheduledFolderSync();
     void slotEtagPollTimerTimeout();
 
-    void slotAccountRemoved(AccountState *accountState);
+    void slotAccountRemoved(OCC::AccountState *accountState);
 
-    void slotRemoveFoldersForAccount(AccountState *accountState);
+    void slotRemoveFoldersForAccount(OCC::AccountState *accountState);
 
     // Wraps the Folder::syncStateChange() signal into the
     // FolderMan::folderSyncStateChange(Folder*) signal.
     void slotForwardFolderSyncStateChange();
 
-    void slotServerVersionChanged(Account *account);
+    void slotServerVersionChanged(OCC::Account *account);
 
     /**
      * A file whose locks were being monitored has become unlocked.
@@ -305,9 +317,9 @@ private slots:
      */
     void slotScheduleFolderByTime();
 
-    void slotSetupPushNotifications(const Folder::Map &);
-    void slotProcessFilesPushNotification(Account *account);
-    void slotConnectToPushNotifications(Account *account);
+    void slotSetupPushNotifications(const OCC::Folder::Map &);
+    void slotProcessFilesPushNotification(OCC::Account *account);
+    void slotConnectToPushNotifications(OCC::Account *account);
 
 private:
     /** Adds a new folder, does not add it to the account settings and
@@ -324,7 +336,7 @@ private:
 
     // finds all folder configuration files
     // and create the folders
-    QString getBackupName(QString fullPathName) const;
+    [[nodiscard]] QString getBackupName(QString fullPathName) const;
 
     // makes the folder known to the socket api
     void registerFolderWithSocketApi(Folder *folder);
@@ -339,7 +351,7 @@ private:
 
     bool pushNotificationsFilesReady(Account *account);
 
-    bool isSwitchToVfsNeeded(const FolderDefinition &folderDefinition) const;
+    [[nodiscard]] bool isSwitchToVfsNeeded(const FolderDefinition &folderDefinition) const;
 
     QSet<Folder *> _disabledFolders;
     Folder::Map _folderMap;
@@ -375,13 +387,13 @@ private:
 
     bool _appRestartRequired = false;
 
-    QMap<QString, QMetaObject::Connection> _localFileEditingSyncFinishedConnections;
-
     static FolderMan *_instance;
     explicit FolderMan(QObject *parent = nullptr);
     friend class OCC::Application;
     friend class ::TestFolderMan;
     friend class ::TestCfApiShellExtensionsIPC;
+    friend class ::ShareTestHelper;
+    friend class ::EndToEndTestHelper;
 };
 
 } // namespace OCC
