@@ -51,6 +51,61 @@
 
 namespace {
 static constexpr char showMainDialogAsNormalWindowC[] = "showMainDialogAsNormalWindow";
+static constexpr char showConfigBackupWarningC[] = "showConfigBackupWarning";
+
+static constexpr char remotePollIntervalC[] = "remotePollInterval";
+static constexpr char forceSyncIntervalC[] = "forceSyncInterval";
+static constexpr char fullLocalDiscoveryIntervalC[] = "fullLocalDiscoveryInterval";
+static constexpr char notificationRefreshIntervalC[] = "notificationRefreshInterval";
+static constexpr char monoIconsC[] = "monoIcons";
+static constexpr char promptDeleteC[] = "promptDeleteAllFiles";
+static constexpr char crashReporterC[] = "crashReporter";
+static constexpr char optionalServerNotificationsC[] = "optionalServerNotifications";
+static constexpr char showCallNotificationsC[] = "showCallNotifications";
+static constexpr char showInExplorerNavigationPaneC[] = "showInExplorerNavigationPane";
+static constexpr char skipUpdateCheckC[] = "skipUpdateCheck";
+static constexpr char autoUpdateCheckC[] = "autoUpdateCheck";
+static constexpr char updateCheckIntervalC[] = "updateCheckInterval";
+static constexpr char updateSegmentC[] = "updateSegment";
+static constexpr char updateChannelC[] = "updateChannel";
+static constexpr char overrideServerUrlC[] = "overrideServerUrl";
+static constexpr char overrideLocalDirC[] = "overrideLocalDir";
+static constexpr char isVfsEnabledC[] = "isVfsEnabled";
+static constexpr char geometryC[] = "geometry";
+static constexpr char timeoutC[] = "timeout";
+static constexpr char chunkSizeC[] = "chunkSize";
+static constexpr char minChunkSizeC[] = "minChunkSize";
+static constexpr char maxChunkSizeC[] = "maxChunkSize";
+static constexpr char targetChunkUploadDurationC[] = "targetChunkUploadDuration";
+static constexpr char automaticLogDirC[] = "logToTemporaryLogDir";
+static constexpr char logDirC[] = "logDir";
+static constexpr char logDebugC[] = "logDebug";
+static constexpr char logExpireC[] = "logExpire";
+static constexpr char logFlushC[] = "logFlush";
+static constexpr char showExperimentalOptionsC[] = "showExperimentalOptions";
+static constexpr char clientVersionC[] = "clientVersion";
+
+static constexpr char proxyHostC[] = "Proxy/host";
+static constexpr char proxyTypeC[] = "Proxy/type";
+static constexpr char proxyPortC[] = "Proxy/port";
+static constexpr char proxyUserC[] = "Proxy/user";
+static constexpr char proxyPassC[] = "Proxy/pass";
+static constexpr char proxyNeedsAuthC[] = "Proxy/needsAuth";
+
+static constexpr char useUploadLimitC[] = "BWLimit/useUploadLimit";
+static constexpr char useDownloadLimitC[] = "BWLimit/useDownloadLimit";
+static constexpr char uploadLimitC[] = "BWLimit/uploadLimit";
+static constexpr char downloadLimitC[] = "BWLimit/downloadLimit";
+
+static constexpr char newBigFolderSizeLimitC[] = "newBigFolderSizeLimit";
+static constexpr char useNewBigFolderSizeLimitC[] = "useNewBigFolderSizeLimit";
+static constexpr char confirmExternalStorageC[] = "confirmExternalStorage";
+static constexpr char moveToTrashC[] = "moveToTrash";
+
+static constexpr char certPath[] = "http_certificatePath";
+static constexpr char certPasswd[] = "http_certificatePasswd";
+
+static const QSet validUpdateChannels { QStringLiteral("stable"), QStringLiteral("beta") };
 }
 
 namespace OCC {
@@ -59,57 +114,8 @@ namespace chrono = std::chrono;
 
 Q_LOGGING_CATEGORY(lcConfigFile, "nextcloud.sync.configfile", QtInfoMsg)
 
-//static const char caCertsKeyC[] = "CaCertificates"; only used from account.cpp
-static const char remotePollIntervalC[] = "remotePollInterval";
-static const char forceSyncIntervalC[] = "forceSyncInterval";
-static const char fullLocalDiscoveryIntervalC[] = "fullLocalDiscoveryInterval";
-static const char notificationRefreshIntervalC[] = "notificationRefreshInterval";
-static const char monoIconsC[] = "monoIcons";
-static const char promptDeleteC[] = "promptDeleteAllFiles";
-static const char crashReporterC[] = "crashReporter";
-static const char optionalServerNotificationsC[] = "optionalServerNotifications";
-static const char showCallNotificationsC[] = "showCallNotifications";
-static const char showInExplorerNavigationPaneC[] = "showInExplorerNavigationPane";
-static const char skipUpdateCheckC[] = "skipUpdateCheck";
-static const char autoUpdateCheckC[] = "autoUpdateCheck";
-static const char updateCheckIntervalC[] = "updateCheckInterval";
-static const char updateSegmentC[] = "updateSegment";
-static const char updateChannelC[] = "updateChannel";
-static const char geometryC[] = "geometry";
-static const char timeoutC[] = "timeout";
-static const char chunkSizeC[] = "chunkSize";
-static const char minChunkSizeC[] = "minChunkSize";
-static const char maxChunkSizeC[] = "maxChunkSize";
-static const char targetChunkUploadDurationC[] = "targetChunkUploadDuration";
-static const char automaticLogDirC[] = "logToTemporaryLogDir";
-static const char logDirC[] = "logDir";
-static const char logDebugC[] = "logDebug";
-static const char logExpireC[] = "logExpire";
-static const char logFlushC[] = "logFlush";
-static const char showExperimentalOptionsC[] = "showExperimentalOptions";
-static const char clientVersionC[] = "clientVersion";
-
-static const char proxyHostC[] = "Proxy/host";
-static const char proxyTypeC[] = "Proxy/type";
-static const char proxyPortC[] = "Proxy/port";
-static const char proxyUserC[] = "Proxy/user";
-static const char proxyPassC[] = "Proxy/pass";
-static const char proxyNeedsAuthC[] = "Proxy/needsAuth";
-
-static const char useUploadLimitC[] = "BWLimit/useUploadLimit";
-static const char useDownloadLimitC[] = "BWLimit/useDownloadLimit";
-static const char uploadLimitC[] = "BWLimit/uploadLimit";
-static const char downloadLimitC[] = "BWLimit/downloadLimit";
-
-static const char newBigFolderSizeLimitC[] = "newBigFolderSizeLimit";
-static const char useNewBigFolderSizeLimitC[] = "useNewBigFolderSizeLimit";
-static const char confirmExternalStorageC[] = "confirmExternalStorage";
-static const char moveToTrashC[] = "moveToTrash";
-
-const char certPath[] = "http_certificatePath";
-const char certPasswd[] = "http_certificatePasswd";
-QString ConfigFile::_confDir = QString();
-bool ConfigFile::_askedUser = false;
+QString ConfigFile::_confDir = {};
+QString ConfigFile::_discoveredLegacyConfigPath = {};
 
 static chrono::milliseconds millisecondsValue(const QSettings &setting, const char *key,
     chrono::milliseconds defaultValue)
@@ -428,28 +434,37 @@ QString ConfigFile::excludeFileFromSystem()
     return fi.absoluteFilePath();
 }
 
-QString ConfigFile::backup() const
+QString ConfigFile::backup(const QString &fileName) const
 {
-    QString baseFile = configFile();
+    const QString baseFilePath = configPath() + fileName;
     auto versionString = clientVersionString();
-    if (!versionString.isEmpty())
+
+    if (!versionString.isEmpty()) {
         versionString.prepend('_');
+    }
+
     QString backupFile =
         QString("%1.backup_%2%3")
-            .arg(baseFile)
+            .arg(baseFilePath)
             .arg(QDateTime::currentDateTime().toString("yyyyMMdd_HHmmss"))
             .arg(versionString);
 
     // If this exact file already exists it's most likely that a backup was
     // already done. (two backup calls directly after each other, potentially
     // even with source alterations in between!)
-    if (!QFile::exists(backupFile)) {
-        QFile f(baseFile);
-        f.copy(backupFile);
+    // QFile does not overwrite backupFile
+    if(!QFile::copy(baseFilePath, backupFile)) {
+        qCWarning(lcConfigFile) << "Failed to create a backup of the config file" << baseFilePath;
     }
+
     return backupFile;
 }
 
+bool ConfigFile::showConfigBackupWarning() const
+{
+    return getValue(showConfigBackupWarningC, QString(), false).toBool();
+}
+
 QString ConfigFile::configFile() const
 {
     return configPath() + Theme::instance()->configFileName();
@@ -679,15 +694,67 @@ QString ConfigFile::updateChannel() const
     }
 
     QSettings settings(configFile(), QSettings::IniFormat);
-    return settings.value(QLatin1String(updateChannelC), defaultUpdateChannel).toString();
+    const auto channel = settings.value(QLatin1String(updateChannelC), defaultUpdateChannel).toString();
+    if (!validUpdateChannels.contains(channel)) {
+        qCWarning(lcConfigFile()) << "Received invalid update channel from confog:"
+                                  << channel
+                                  << "defaulting to:"
+                                  << defaultUpdateChannel;
+        return defaultUpdateChannel;
+    }
+
+    return channel;
 }
 
 void ConfigFile::setUpdateChannel(const QString &channel)
 {
+    if (!validUpdateChannels.contains(channel)) {
+        qCWarning(lcConfigFile()) << "Received invalid update channel:"
+                                  << channel
+                                  << "can only accept 'stable' or 'beta'. Ignoring.";
+        return;
+    }
+
     QSettings settings(configFile(), QSettings::IniFormat);
     settings.setValue(QLatin1String(updateChannelC), channel);
 }
 
+[[nodiscard]] QString ConfigFile::overrideServerUrl() const
+{
+    QSettings settings(configFile(), QSettings::IniFormat);
+    return settings.value(QLatin1String(overrideServerUrlC), {}).toString();
+}
+
+void ConfigFile::setOverrideServerUrl(const QString &url)
+{
+    QSettings settings(configFile(), QSettings::IniFormat);
+    settings.setValue(QLatin1String(overrideServerUrlC), url);
+}
+
+[[nodiscard]] QString ConfigFile::overrideLocalDir() const
+{
+    QSettings settings(configFile(), QSettings::IniFormat);
+    return settings.value(QLatin1String(overrideLocalDirC), {}).toString();
+}
+
+void ConfigFile::setOverrideLocalDir(const QString &localDir)
+{
+    QSettings settings(configFile(), QSettings::IniFormat);
+    settings.setValue(QLatin1String(overrideLocalDirC), localDir);
+}
+
+bool ConfigFile::isVfsEnabled() const
+{
+    QSettings settings(configFile(), QSettings::IniFormat);
+    return settings.value({isVfsEnabledC}, {}).toBool();
+}
+
+void ConfigFile::setVfsEnabled(bool enabled)
+{
+    QSettings settings(configFile(), QSettings::IniFormat);
+    settings.setValue({isVfsEnabledC}, enabled);
+}
+
 void ConfigFile::setProxyType(int proxyType,
     const QString &host,
     int port, bool needsAuth,
@@ -1089,4 +1156,19 @@ void ConfigFile::setupDefaultExcludeFilePaths(ExcludedFiles &excludedFiles)
         excludedFiles.addExcludeFilePath(userList);
     }
 }
+
+QString ConfigFile::discoveredLegacyConfigPath()
+{
+    return _discoveredLegacyConfigPath;
+}
+
+void ConfigFile::setDiscoveredLegacyConfigPath(const QString &discoveredLegacyConfigPath)
+{
+    if (_discoveredLegacyConfigPath == discoveredLegacyConfigPath) {
+        return;
+    }
+
+    _discoveredLegacyConfigPath = discoveredLegacyConfigPath;
+}
+
 }
